require "rbacfull/version"


module Rbacfull
  extend Configure
  attr_accessor :role_cname, :adapter, :resource_adapter, :role_join_table_name, :role_table_name
    @@resource_types = []

    def rbacfull(options = {})
      include Role
      extend Dynamic if Rbacfull.dynamic_shortcuts

      options.reverse_merge!({:role_cname => 'Role'})
      self.role_cname = options[:role_cname]
      self.role_table_name = self.role_cname.tableize.gsub(/\//, "_")

      default_join_table = "#{self.to_s.tableize.gsub(/\//, "_")}_#{self.role_table_name}"
      options.reverse_merge!({:role_join_table_name => default_join_table})
      self.role_join_table_name = options[:role_join_table_name]

      rbacfull_options = { :class_name => options[:role_cname].camelize }
      rbacfull_options.merge!({ :join_table => self.role_join_table_name }) if Rbacfull.orm == "active_record"
      rbacfull_options.merge!(options.reject{ |k,v| ![ :before_add, :after_add, :before_remove, :after_remove ].include? k.to_sym })

      has_and_belongs_to_many :roles, rbacfull_options

      self.adapter = Rbacfull::Adapter::Base.create("role_adapter", self.role_cname, self.name)
      load_dynamic_methods if Rbacfull.dynamic_shortcuts
    end

    def adapter
      return self.superclass.adapter unless self.instance_variable_defined? '@adapter'
      @adapter
    end

    def resourcify(association_name = :roles, options = {})
      include Resource

      options.reverse_merge!({ :role_cname => 'Role', :dependent => :destroy })
      resourcify_options = { :class_name => options[:role_cname].camelize, :as => :resource, :dependent => options[:dependent] }
      self.role_cname = options[:role_cname]
      self.role_table_name = self.role_cname.tableize.gsub(/\//, "_")

      has_many association_name, resourcify_options

      self.resource_adapter = Rbacfull::Adapter::Base.create("resource_adapter", self.role_cname, self.name)
      @@resource_types << self.name
    end

    def resource_adapter
      return self.superclass.resource_adapter unless self.instance_variable_defined? '@resource_adapter'
      @resource_adapter
    end

    def scopify
      require "rbacfull/adapters/#{Rbacfull.orm}/scopes.rb"
      extend Rbacfull::Adapter::Scopes
    end

    def role_class
      return self.superclass.role_class unless self.instance_variable_defined? '@role_cname'
      self.role_cname.constantize
    end

    def self.resource_types
      @@resource_types
    end
	end
